@@ -26,6 +26,7 @@ type backupOpts struct {
 	compressionLevel []int
 	ns               string
 	wait             bool
+	externList       bool
 }
 
 type backupOut struct {
@@ -37,6 +38,32 @@ func (b backupOut) String() string {
 	return fmt.Sprintf("Backup '%s' to remote store '%s' has started", b.Name, b.Storage)
 }
 
+type externBcpOut struct {
+	Name  string          `json:"name"`
+	Nodes []externBcpNode `json:"storage"`
+
+	list bool
+}
+
+type externBcpNode struct {
+	Name  string   `json:"name"`
+	Files []string `json:"files"`
+}
+
+func (b externBcpOut) String() string {
+	s := fmt.Sprintln("Ready to copy data from:")
+	for _, n := range b.Nodes {
+		s += fmt.Sprintf("\t- %s\n", n.Name)
+		if b.list {
+			for _, f := range n.Files {
+				s += fmt.Sprintf("\t\t%s\n", f)
+			}
+		}
+	}
+	s += fmt.Sprintf("After the copy is done, run: pbm backup-finish %s\n", b.Name)
+	return s
+}
+
 type descBcp struct {
 	name string
 }
@@ -49,8 +76,16 @@ func runBackup(cn *pbm.PBM, b *backupOpts, outf outFormat) (fmt.Stringer, error)
 	if len(nss) > 1 {
 		return nil, errors.New("parse --ns option: multiple namespaces are not supported")
 	}
-	if len(nss) != 0 && b.typ == string(pbm.PhysicalBackup) {
-		return nil, errors.New("--ns flag is not allowed for physical backup")
+	if len(nss) != 0 && b.typ != string(pbm.LogicalBackup) {
+		return nil, errors.New("--ns flag is only allowed for logical backup")
+	}
+
+	ver, err := pbm.GetMongoVersion(cn.Context(), cn.Conn)
+	if err != nil {
+		return nil, errors.WithMessage(err, "get mongo version")
+	}
+	if err = pbm.FeatureSupport(ver).BackupType(pbm.BackupType(b.typ)); err != nil {
+		return nil, errors.WithMessage(err, "unsupported backup type")
 	}
 
 	if err := checkConcurrentOp(cn); err != nil {
@@ -106,40 +141,81 @@ func runBackup(cn *pbm.PBM, b *backupOpts, outf outFormat) (fmt.Stringer, error)
 		return nil, err
 	}
 
+	if b.typ == string(pbm.ExternalBackup) {
+		s, err := waitBackup(context.Background(), cn, b.name, pbm.StatusCopyReady)
+		if err != nil {
+			return nil, errors.Wrap(err, "waiting for the `copyReady` status")
+		}
+		if s == nil || *s != pbm.StatusCopyReady {
+			str := "nil"
+			if s != nil {
+				str = string(*s)
+			}
+			return nil, errors.Errorf("unexpected backup status %v", str)
+		}
+
+		bcp, err := cn.GetBackupMeta(b.name)
+		if err != nil {
+			return nil, errors.Wrap(err, "get backup meta")
+		}
+		out := externBcpOut{Name: b.name, list: b.externList}
+		for _, rs := range bcp.Replsets {
+			node := externBcpNode{Name: rs.Node}
+			for _, f := range rs.Files {
+				node.Files = append(node.Files, f.Name)
+			}
+			out.Nodes = append(out.Nodes, node)
+		}
+		return out, nil
+	}
+
 	if b.wait {
-		return outMsg{}, waitBackup(context.Background(), cn, b.name)
+		fmt.Printf("\nWaiting for '%s' backup...", b.name)
+		s, err := waitBackup(context.Background(), cn, b.name, pbm.StatusDone)
+		if s != nil {
+			fmt.Printf(" %s\n", *s)
+		}
+		return outMsg{}, err
 	}
 
-	fmt.Println()
 	return backupOut{b.name, cfg.Storage.Path()}, nil
 }
 
-func waitBackup(ctx context.Context, cn *pbm.PBM, name string) error {
+func runFinishBcp(cn *pbm.PBM, bcp string) (fmt.Stringer, error) {
+	meta, err := cn.GetBackupMeta(bcp)
+	if err != nil {
+		if errors.Is(err, pbm.ErrNotFound) {
+			return nil, errors.Errorf("backup %q not found", bcp)
+		}
+		return nil, err
+	}
+	if meta.Status != pbm.StatusCopyReady {
+		return nil, errors.Errorf("expected %q status. got %q", pbm.StatusCopyReady, meta.Status)
+	}
+
+	return outMsg{fmt.Sprintf("Command sent. Check `pbm describe-backup %s` for the result.", bcp)},
+		cn.ChangeBackupState(bcp, pbm.StatusCopyDone, "")
+}
+
+func waitBackup(ctx context.Context, cn *pbm.PBM, name string, status pbm.Status) (*pbm.Status, error) {
 	t := time.NewTicker(time.Second)
 	defer t.Stop()
 
-	fmt.Printf("\nWaiting for '%s' backup...", name)
-
 	for {
 		select {
 		case <-ctx.Done():
-			return ctx.Err()
+			return nil, ctx.Err()
 		case <-t.C:
 			bcp, err := cn.GetBackupMeta(name)
 			if err != nil {
-				return err
+				return nil, err
 			}
 
 			switch bcp.Status {
-			case pbm.StatusDone:
-				fmt.Println(" done")
-				return nil
-			case pbm.StatusCancelled:
-				fmt.Println(" canceled")
-				return nil
+			case status, pbm.StatusDone, pbm.StatusCancelled:
+				return &bcp.Status, nil
 			case pbm.StatusError:
-				fmt.Println(" failed")
-				return bcp.Error()
+				return &bcp.Status, bcp.Error()
 			}
 		}
 
@@ -218,6 +294,8 @@ type bcpDesc struct {
 type bcpReplDesc struct {
 	Name               string             `json:"name" yaml:"name"`
 	Status             pbm.Status         `json:"status" yaml:"status"`
+	Node               string             `json:"node" yaml:"node"`
+	Files              []pbm.File         `json:"files,omitempty" yaml:"-"`
 	LastWriteTS        int64              `json:"last_write_ts" yaml:"-"`
 	LastTransitionTS   int64              `json:"last_transition_ts" yaml:"-"`
 	LastWriteTime      string             `json:"last_write_time" yaml:"last_write_time"`
@@ -298,6 +376,7 @@ func describeBackup(cn *pbm.PBM, b *descBcp) (fmt.Stringer, error) {
 	for i, r := range bcp.Replsets {
 		rv.Replsets[i] = bcpReplDesc{
 			Name:               r.Name,
+			Node:               r.Node,
 			IsConfigSvr:        r.IsConfigSvr,
 			Status:             r.Status,
 			LastWriteTS:        int64(r.LastWriteTS.T),
@@ -311,6 +390,9 @@ func describeBackup(cn *pbm.PBM, b *descBcp) (fmt.Stringer, error) {
 		if r.MongodOpts != nil && r.MongodOpts.Security != nil {
 			rv.Replsets[i].SecurityOpts = r.MongodOpts.Security
 		}
+		if bcp.Type == pbm.ExternalBackup {
+			rv.Replsets[i].Files = r.Files
+		}
 	}
 
 	return rv, err
@@ -333,13 +415,7 @@ func bcpsMatchCluster(bcps []pbm.BackupMeta, ver, fcv string, shards []pbm.Shard
 
 	mapRS, mapRevRS := pbm.MakeRSMapFunc(rsMap), pbm.MakeReverseRSMapFunc(rsMap)
 	for i := 0; i < len(bcps); i++ {
-		bcp := &bcps[i]
-		if bcps[i].Type == pbm.PhysicalBackup && len(rsMap) != 0 {
-			bcp.SetRuntimeError(errRSMappingWithPhysBackup{})
-			continue
-		}
-
-		bcpMatchCluster(bcp, ver, fcv, sh, mapRS, mapRevRS)
+		bcpMatchCluster(&bcps[i], ver, fcv, sh, mapRS, mapRevRS)
 	}
 }
 
@@ -348,18 +424,16 @@ func bcpMatchCluster(bcp *pbm.BackupMeta, ver, fcv string, shards map[string]boo
 		return
 	}
 	if !version.CompatibleWith(bcp.PBMVersion, pbm.BreakingChangesMap[bcp.Type]) {
-		bcp.SetRuntimeError(errIncompatibleVersion{bcp.PBMVersion})
+		bcp.SetRuntimeError(errIncompatiblePBMVersion{bcp.PBMVersion})
 		return
 	}
 	if bcp.FCV != "" {
 		if bcp.FCV != fcv {
-			bcp.SetRuntimeError(errors.Errorf("backup FCV %q is incompatible with the running mongo FCV %q",
-				bcp.FCV, fcv))
+			bcp.SetRuntimeError(errIncompatibleFCVVersion{bcp.FCV, fcv})
 			return
 		}
 	} else if majmin(bcp.MongoVersion) != majmin(ver) {
-		bcp.SetRuntimeError(errors.Errorf("backup mongo version %q is incompatible with the running mongo version %q",
-			bcp.MongoVersion, ver))
+		bcp.SetRuntimeError(errIncompatibleMongodVersion{bcp.MongoVersion, ver})
 		return
 
 	}
@@ -402,16 +476,6 @@ func majmin(v string) string {
 
 var errIncompatible = errors.New("incompatible")
 
-type errRSMappingWithPhysBackup struct{}
-
-func (errRSMappingWithPhysBackup) Error() string {
-	return "unsupported with replset remapping"
-}
-
-func (errRSMappingWithPhysBackup) Unwrap() error {
-	return errIncompatible
-}
-
 type errMissedReplsets struct {
 	names     []string
 	configsrv bool
@@ -439,15 +503,41 @@ func (errMissedReplsets) Unwrap() error {
 	return errIncompatible
 }
 
-type errIncompatibleVersion struct {
+type errIncompatiblePBMVersion struct {
 	bcpVer string
 }
 
-func (e errIncompatibleVersion) Unwrap() error {
+func (e errIncompatiblePBMVersion) Unwrap() error {
 	return errIncompatible
 }
 
-func (e errIncompatibleVersion) Error() string {
+func (e errIncompatiblePBMVersion) Error() string {
 	return fmt.Sprintf("backup version (v%s) is not compatible with PBM v%s",
 		e.bcpVer, version.DefaultInfo.Version)
 }
+
+type errIncompatibleFCVVersion struct {
+	bcpVer  string
+	currVer string
+}
+
+func (e errIncompatibleFCVVersion) Unwrap() error {
+	return errIncompatible
+}
+
+func (e errIncompatibleFCVVersion) Error() string {
+	return fmt.Sprintf("backup FCV %q is incompatible with the running mongo FCV %q", e.bcpVer, e.currVer)
+}
+
+type errIncompatibleMongodVersion struct {
+	bcpVer  string
+	currVer string
+}
+
+func (e errIncompatibleMongodVersion) Unwrap() error {
+	return errIncompatible
+}
+
+func (e errIncompatibleMongodVersion) Error() string {
+	return fmt.Sprintf("backup mongo version %q is incompatible with the running mongo version %q", majmin(e.bcpVer), majmin(e.currVer))
+}
