@@ -3,7 +3,9 @@ package main
 import (
 	"fmt"
 	"log"
+	"math/rand"
 	"os"
+	"time"
 
 	"go.mongodb.org/mongo-driver/bson/primitive"
 	"golang.org/x/mod/semver"
@@ -27,9 +29,19 @@ func run(t *sharded.Cluster, typ testTyp) {
 		{"GCS", "/etc/pbm/gcs.yaml"},
 		{"Azure", "/etc/pbm/azure.yaml"},
 		{"FS", "/etc/pbm/fs.yaml"},
-		{"Minio", "/etc/pbm/minio.yaml"},
 	}
 
+	rand.Seed(time.Now().UnixNano())
+	rand.Shuffle(len(remoteStg), func(i, j int) {
+		remoteStg[i], remoteStg[j] = remoteStg[j], remoteStg[i]
+	})
+
+	minio := struct {
+		name string
+		conf string
+	}{name: "Minio", conf: "/etc/pbm/minio.yaml"}
+	remoteStg = append(remoteStg, minio)
+
 	for _, stg := range remoteStg {
 		if confExt(stg.conf) {
 			storage = stg.conf
