@@ -69,7 +69,6 @@ const (
 	CmdCancelBackup Command = "cancelBackup"
 	CmdResync       Command = "resync"
 	CmdPITR         Command = "pitr"
-	CmdPITRestore   Command = "pitrestore"
 	CmdDeleteBackup Command = "delete"
 	CmdDeletePITR   Command = "deletePitr"
 	CmdCleanup      Command = "cleanup"
@@ -89,8 +88,6 @@ func (c Command) String() string {
 		return "Resync storage"
 	case CmdPITR:
 		return "PITR incremental backup"
-	case CmdPITRestore:
-		return "PITR restore"
 	case CmdDeleteBackup:
 		return "Delete"
 	case CmdDeletePITR:
@@ -109,7 +106,6 @@ type Cmd struct {
 	Backup     *BackupCmd       `bson:"backup,omitempty"`
 	Restore    *RestoreCmd      `bson:"restore,omitempty"`
 	Replay     *ReplayCmd       `bson:"replay,omitempty"`
-	PITRestore *PITRestoreCmd   `bson:"pitrestore,omitempty"`
 	Delete     *DeleteBackupCmd `bson:"delete,omitempty"`
 	DeletePITR *DeletePITRCmd   `bson:"deletePitr,omitempty"`
 	Cleanup    *CleanupCmd      `bson:"cleanup,omitempty"`
@@ -148,10 +144,6 @@ func (c Cmd) String() string {
 		buf.WriteString(" [")
 		buf.WriteString(c.Restore.String())
 		buf.WriteString("]")
-	case CmdPITRestore:
-		buf.WriteString(" [")
-		buf.WriteString(c.PITRestore.String())
-		buf.WriteString("]")
 	}
 	buf.WriteString(" <ts: ")
 	buf.WriteString(strconv.FormatInt(c.TS, 10))
@@ -183,10 +175,31 @@ type RestoreCmd struct {
 	BackupName string            `bson:"backupName"`
 	Namespaces []string          `bson:"nss,omitempty"`
 	RSMap      map[string]string `bson:"rsMap,omitempty"`
+
+	OplogTS primitive.Timestamp `bson:"oplogTS,omitempty"`
+
+	External bool                `bson:"external"`
+	ExtConf  ExternOpts          `bson:"extConf"`
+	ExtTS    primitive.Timestamp `bson:"extTS"`
 }
 
 func (r RestoreCmd) String() string {
-	return fmt.Sprintf("name: %s, backup name: %s", r.Name, r.BackupName)
+	bcp := ""
+	if r.BackupName != "" {
+		bcp = "snapshot: " + r.BackupName
+	}
+	if r.External {
+		bcp += "[external]"
+	}
+	if r.ExtTS.T > 0 {
+		bcp += fmt.Sprintf(" external ts: <%d,%d>", r.ExtTS.T, r.ExtTS.I)
+	}
+	if r.OplogTS.T > 0 {
+		bcp += fmt.Sprintf(" point-in-time: <%d,%d>", r.OplogTS.T, r.OplogTS.I)
+	}
+
+	return fmt.Sprintf("name: %s, %s", r.Name, bcp)
+
 }
 
 type ReplayCmd struct {
@@ -200,22 +213,6 @@ func (c ReplayCmd) String() string {
 	return fmt.Sprintf("name: %s, time: %d - %d", c.Name, c.Start, c.End)
 }
 
-type PITRestoreCmd struct {
-	Name       string            `bson:"name"`
-	TS         int64             `bson:"ts"`
-	I          int64             `bson:"i"`
-	Bcp        string            `bson:"bcp"`
-	Namespaces []string          `bson:"nss,omitempty"`
-	RSMap      map[string]string `bson:"rsMap,omitempty"`
-}
-
-func (p PITRestoreCmd) String() string {
-	if p.Bcp != "" {
-		return fmt.Sprintf("name: %s, point-in-time ts: %d, base-snapshot: %s", p.Name, p.TS, p.Bcp)
-	}
-	return fmt.Sprintf("name: %s, point-in-time ts: %d", p.Name, p.TS)
-}
-
 type DeleteBackupCmd struct {
 	Backup    string `bson:"backup"`
 	OlderThan int64  `bson:"olderthan"`
@@ -465,6 +462,7 @@ type BackupType string
 
 const (
 	PhysicalBackup    BackupType = "physical"
+	ExternalBackup    BackupType = "external"
 	IncrementalBackup BackupType = "incremental"
 	LogicalBackup     BackupType = "logical"
 )
@@ -480,6 +478,11 @@ type BackupMeta struct {
 	// Empty means this is a full backup (and a base for further incremental bcps).
 	SrcBackup string `bson:"src_backup,omitempty" json:"src_backup,omitempty"`
 
+	// ShardRemap is map of replset to shard names.
+	// If shard name is different from replset name, it will be stored in the map.
+	// If all shard names are the same as their replset names, the map is nil.
+	ShardRemap map[string]string `bson:"shardRemap,omitempty" json:"shardRemap,omitempty"`
+
 	Namespaces       []string                 `bson:"nss,omitempty" json:"nss,omitempty"`
 	Replsets         []BackupReplset          `bson:"replsets" json:"replsets"`
 	Compression      compress.CompressionType `bson:"compression" json:"compression"`
@@ -592,10 +595,16 @@ const (
 	StatusStarting   Status = "starting"
 	StatusRunning    Status = "running"
 	StatusDumpDone   Status = "dumpDone"
+	StatusCopyReady  Status = "copyReady"
+	StatusCopyDone   Status = "copyDone"
 	StatusPartlyDone Status = "partlyDone"
 	StatusDone       Status = "done"
 	StatusCancelled  Status = "canceled"
 	StatusError      Status = "error"
+
+	// status to communicate last op timestamp if it's not set
+	// during external restore
+	StatusExtTS Status = "lastTS"
 )
 
 func (p *PBM) SetBackupMeta(m *BackupMeta) error {
