@@ -19,6 +19,7 @@ import (
 	"sync"
 	"time"
 
+	"github.com/mongodb/mongo-tools/common/db"
 	"github.com/pkg/errors"
 	"go.mongodb.org/mongo-driver/bson"
 	"go.mongodb.org/mongo-driver/bson/primitive"
@@ -66,12 +67,13 @@ type PhysRestore struct {
 	startTS int64
 	secOpts *pbm.MongodOptsSec
 
-	name     string
-	opid     string
-	nodeInfo *pbm.NodeInfo
-	stg      storage.Storage
-	bcp      *pbm.BackupMeta
-	files    []files
+	name      string
+	opid      string
+	nodeInfo  *pbm.NodeInfo
+	stg       storage.Storage
+	bcp       *pbm.BackupMeta
+	files     []files
+	restoreTS primitive.Timestamp
 
 	confOpts pbm.RestoreConf
 
@@ -85,7 +87,6 @@ type PhysRestore struct {
 	syncPathCluster  string
 	syncPathPeers    map[string]struct{}
 	// Shards to participate in restore.
-	// Only the restore leader would have this info.
 	syncPathShards map[string]struct{}
 	// Non-ConfigServer shards
 	syncPathDataShards map[string]struct{}
@@ -93,9 +94,11 @@ type PhysRestore struct {
 	stopHB chan struct{}
 
 	log *log.Event
+
+	rsMap map[string]string
 }
 
-func NewPhysical(cn *pbm.PBM, node *pbm.Node, inf *pbm.NodeInfo) (*PhysRestore, error) {
+func NewPhysical(cn *pbm.PBM, node *pbm.Node, inf *pbm.NodeInfo, rsMap map[string]string) (*PhysRestore, error) {
 	opts, err := node.GetOpts(nil)
 	if err != nil {
 		return nil, errors.Wrap(err, "get mongo options")
@@ -121,15 +124,22 @@ func NewPhysical(cn *pbm.PBM, node *pbm.Node, inf *pbm.NodeInfo) (*PhysRestore,
 
 	var shards map[string]string
 	var csvr string
-	if inf.IsConfigSrv() {
-		shards, err = node.GetShardsConfig()
+	if inf.IsSharded() {
+		ss, err := cn.GetShards()
 		if err != nil {
-			return nil, errors.Wrap(err, "get shards list")
+			return nil, errors.WithMessage(err, "get shards")
 		}
-	} else if inf.IsSharded() {
-		csvr, err = node.ConfSvrConn()
-		if err != nil {
-			return nil, errors.Wrap(err, "get configsvrConnectionString")
+
+		shards = make(map[string]string)
+		for _, s := range ss {
+			shards[s.ID] = s.Host
+		}
+
+		if inf.ReplsetRole() != pbm.RoleConfigSrv {
+			csvr, err = node.ConfSvrConn()
+			if err != nil {
+				return nil, errors.Wrap(err, "get configsvrConnectionString")
+			}
 		}
 	}
 
@@ -152,6 +162,7 @@ func NewPhysical(cn *pbm.PBM, node *pbm.Node, inf *pbm.NodeInfo) (*PhysRestore,
 		nodeInfo: inf,
 		tmpPort:  tmpPort,
 		secOpts:  opts.Security,
+		rsMap:    rsMap,
 	}, nil
 }
 
@@ -191,7 +202,12 @@ func (r *PhysRestore) close(noerr, cleanup bool) {
 		r.log.Debug("rm tmp logs")
 		err := os.Remove(path.Join(r.dbpath, internalMongodLog))
 		if err != nil {
-			r.log.Error("remove tmp mongod logs %s: %v", path.Join(r.dbpath, internalMongodLog), err)
+			r.log.Warning("remove tmp mongod logs %s: %v", path.Join(r.dbpath, internalMongodLog), err)
+		}
+		extMeta := filepath.Join(r.dbpath, fmt.Sprintf(pbm.ExternalRsMetaFile, pbm.MakeReverseRSMapFunc(r.rsMap)(r.nodeInfo.SetName)))
+		err = os.Remove(extMeta)
+		if err != nil && err != os.ErrNotExist {
+			r.log.Warning("remove external rs meta <%s>: %v", extMeta, err)
 		}
 	} else if cleanup { // clean-up dbpath on err if needed
 		r.log.Debug("clean-up dbpath")
@@ -392,7 +408,7 @@ func (r *PhysRestore) toState(status pbm.Status) (rStatus pbm.Status, err error)
 	r.log.Info("waiting for cluster")
 	cstat, err := r.waitFiles(status, map[string]struct{}{r.syncPathCluster: {}}, true)
 	if err != nil {
-		return pbm.StatusError, errors.Wrap(err, "wait for shards")
+		return pbm.StatusError, errors.Wrap(err, "wait for cluster")
 	}
 
 	r.log.Debug("converged to state %s", cstat)
@@ -400,6 +416,38 @@ func (r *PhysRestore) toState(status pbm.Status) (rStatus pbm.Status, err error)
 	return cstat, nil
 }
 
+func (r *PhysRestore) getTSFromSyncFile(path string) (primitive.Timestamp, error) {
+	res, err := r.stg.SourceReader(path + "." + string(pbm.StatusExtTS))
+	if err != nil {
+		return primitive.Timestamp{}, errors.Wrap(err, "get timestamp")
+	}
+	b, err := io.ReadAll(res)
+	if err != nil {
+		return primitive.Timestamp{}, errors.Wrap(err, "read timestamp")
+	}
+	tsb := bytes.Split(b, []byte(":"))
+	if len(tsb) != 2 {
+		return primitive.Timestamp{}, errors.Errorf("wrong file format: %s", tsb)
+	}
+	tsparts := bytes.Split(tsb[1], []byte(","))
+	if len(tsparts) != 2 {
+		return primitive.Timestamp{}, errors.Errorf("wrong timestamp format: %s", tsparts)
+	}
+	ctsT, err := strconv.Atoi(string(tsparts[0]))
+	if err != nil {
+		return primitive.Timestamp{}, errors.Wrap(err, "parse ts.T")
+	}
+	ctsI, err := strconv.Atoi(string(tsparts[1]))
+	if err != nil {
+		return primitive.Timestamp{}, errors.Wrap(err, "parse ts.I")
+	}
+
+	return primitive.Timestamp{
+		T: uint32(ctsT),
+		I: uint32(ctsI),
+	}, nil
+}
+
 func errStatus(err error) io.Reader {
 	return bytes.NewReader([]byte(
 		fmt.Sprintf("%d:%v", time.Now().Unix(), err),
@@ -579,6 +627,7 @@ func (l *logBuff) flush() error {
 
 	return nil
 }
+
 func (l *logBuff) Flush() error {
 	l.mx.Lock()
 	defer l.mx.Unlock()
@@ -605,7 +654,15 @@ func (l *logBuff) Flush() error {
 //   - Starts standalone mongod to recover oplog from journals.
 //   - Cleans up data and resets replicaset config to the working state.
 //   - Shuts down mongod and agent (the leader also dumps metadata to the storage).
-func (r *PhysRestore) Snapshot(cmd *pbm.RestoreCmd, opid pbm.OPID, l *log.Event, stopAgentC chan<- struct{}, pauseHB func()) (err error) {
+//
+// An External restore doesn't need to have specified backup. It will look
+// for the replset metadata in the datadir after data is copied. And take
+// all needed inputs from there (restoreTS, files list, and mongod config).
+// The user may also specify restoreTS and a mongod config via CLI. The priority:
+// - CLI provided values
+// - replset metada in the datadir
+// - backup meta
+func (r *PhysRestore) Snapshot(cmd *pbm.RestoreCmd, pitr primitive.Timestamp, opid pbm.OPID, l *log.Event, stopAgentC chan<- struct{}, pauseHB func()) (err error) {
 	l.Debug("port: %d", r.tmpPort)
 
 	meta := &pbm.RestoreMeta{
@@ -637,14 +694,32 @@ func (r *PhysRestore) Snapshot(cmd *pbm.RestoreCmd, opid pbm.OPID, l *log.Event,
 		return errors.Wrap(err, "init")
 	}
 
-	err = r.prepareBackup(cmd.BackupName)
-	if err != nil {
-		return err
+	if cmd.BackupName == "" && !cmd.External {
+		return errors.New("restore isn't external and no backup set")
 	}
-	meta.Type = r.bcp.Type
-	err = r.setTmpConf()
-	if err != nil {
-		return errors.Wrap(err, "set tmp config")
+
+	if cmd.BackupName != "" {
+		err = r.prepareBackup(cmd.BackupName)
+		if err != nil {
+			return err
+		}
+		r.restoreTS = r.bcp.LastWriteTS
+	}
+	if cmd.ExtTS.T > 0 {
+		r.restoreTS = cmd.ExtTS
+	}
+	if cmd.External {
+		meta.Type = pbm.ExternalBackup
+	} else {
+		meta.Type = r.bcp.Type
+	}
+
+	var opChunks []pbm.OplogChunk
+	if !pitr.IsZero() {
+		opChunks, err = chunks(r.cn, r.stg, r.restoreTS, pitr, r.rsConf.ID, r.rsMap)
+		if err != nil {
+			return err
+		}
 	}
 
 	if meta.Type == pbm.IncrementalBackup {
@@ -704,14 +779,68 @@ func (r *PhysRestore) Snapshot(cmd *pbm.RestoreCmd, opid pbm.OPID, l *log.Event,
 	// own (which sets the no-return point).
 	progress |= restoreStared
 
-	l.Info("copying backup data")
-	dstat, err := r.copyFiles()
-	if err != nil {
-		return errors.Wrap(err, "copy files")
+	var excfg *pbm.MongodOpts
+	var stats pbm.RestoreShardStat
+
+	if cmd.External {
+		_, err = r.toState(pbm.StatusCopyReady)
+		if err != nil {
+			return errors.Wrapf(err, "moving to state %s", pbm.StatusCopyReady)
+		}
+
+		l.Info("waiting for the datadir to be copied")
+		_, err := r.waitFiles(pbm.StatusCopyDone, map[string]struct{}{r.syncPathCluster: {}}, true)
+		if err != nil {
+			return errors.Wrapf(err, "check %s state", pbm.StatusCopyDone)
+		}
+
+		// try to read replset meta from the backup and use its data
+		setName := pbm.MakeReverseRSMapFunc(r.rsMap)(r.nodeInfo.SetName)
+		rsMetaF := filepath.Join(r.dbpath, fmt.Sprintf(pbm.ExternalRsMetaFile, setName))
+		conff, err := os.Open(rsMetaF)
+		var needFiles []pbm.File
+		if err == nil {
+			rsMeta := new(pbm.BackupReplset)
+			err := json.NewDecoder(conff).Decode(rsMeta)
+			if err != nil {
+				return errors.Wrap(err, "decode replset meta from the backup")
+			}
+			l.Debug("got rs meta from the backup")
+			if r.restoreTS.T == 0 {
+				r.restoreTS = rsMeta.LastWriteTS
+			}
+			excfg = rsMeta.MongodOpts
+			needFiles = rsMeta.Files
+		} else {
+			l.Info("open replset metadata file <%s>: %v. Continue without.", rsMetaF, err)
+		}
+
+		err = r.cleanupDatadir(needFiles)
+		if err != nil {
+			return errors.Wrap(err, "cleanup datadir")
+		}
+	} else {
+		l.Info("copying backup data")
+		stats.D, err = r.copyFiles()
+		if err != nil {
+			return errors.Wrap(err, "copy files")
+		}
 	}
-	err = r.writeStat(dstat)
+
+	if o, ok := cmd.ExtConf[r.nodeInfo.SetName]; ok {
+		excfg = &o
+	}
+	err = r.setTmpConf(excfg)
 	if err != nil {
-		r.log.Warning("write download stat: %v", err)
+		return errors.Wrap(err, "set tmp config")
+	}
+
+	if r.restoreTS.T == 0 {
+		l.Info("restore timestamp isn't set, get latest common ts for the cluster")
+		r.restoreTS, err = r.agreeCommonRestoreTS()
+		if err != nil {
+			return errors.Wrap(err, "get common restore timestamp")
+		}
 	}
 
 	l.Info("preparing data")
@@ -726,6 +855,14 @@ func (r *PhysRestore) Snapshot(cmd *pbm.RestoreCmd, opid pbm.OPID, l *log.Event,
 		return errors.Wrap(err, "recover oplog as standalone")
 	}
 
+	if !pitr.IsZero() && r.nodeInfo.IsPrimary {
+		l.Info("replaying pitr oplog")
+		err = r.replayOplog(r.bcp.LastWriteTS, pitr, opChunks, &stats)
+		if err != nil {
+			return errors.Wrap(err, "replay pitr oplog")
+		}
+	}
+
 	l.Info("clean-up and reset replicaset config")
 	err = r.resetRS()
 	if err != nil {
@@ -743,6 +880,11 @@ func (r *PhysRestore) Snapshot(cmd *pbm.RestoreCmd, opid pbm.OPID, l *log.Event,
 		return errors.Wrapf(err, "moving to state %s", pbm.StatusDone)
 	}
 
+	err = r.writeStat(stats)
+	if err != nil {
+		r.log.Warning("write download stat: %v", err)
+	}
+
 	r.log.Info("writing restore meta")
 	err = r.dumpMeta(meta, stat, "")
 	if err != nil {
@@ -752,13 +894,62 @@ func (r *PhysRestore) Snapshot(cmd *pbm.RestoreCmd, opid pbm.OPID, l *log.Event,
 	return nil
 }
 
-func (r *PhysRestore) writeStat(stat any) error {
-	d := struct {
-		D any `json:"d"`
-	}{
-		D: stat,
+var rmFromDatadir = map[string]struct{}{
+	"WiredTiger.lock":    {},
+	"WiredTiger.turtle":  {},
+	"WiredTiger.wt":      {},
+	"mongod.lock":        {},
+	"ongoingBackup.lock": {},
+}
+
+// removes obsolete files from the datadir
+func (r *PhysRestore) cleanupDatadir(bcpFiles []pbm.File) error {
+	var rm func(f string) bool
+
+	needFiles := bcpFiles
+	if needFiles == nil && r.bcp != nil {
+		rs := getRS(r.bcp, r.nodeInfo.SetName)
+		if rs != nil {
+			needFiles = rs.Files
+		}
 	}
-	b, err := json.Marshal(d)
+
+	if needFiles != nil {
+		m := make(map[string]struct{})
+		for _, f := range needFiles {
+			m[f.Name] = struct{}{}
+		}
+		rm = func(f string) bool {
+			_, ok := m[f]
+			return !ok
+		}
+	} else {
+		rm = func(f string) bool {
+			_, ok := rmFromDatadir[f]
+			return ok
+		}
+	}
+
+	dbpath := path.Clean(r.dbpath) + "/"
+	return filepath.Walk(dbpath, func(p string, info os.FileInfo, err error) error {
+		if err != nil {
+			return errors.Wrap(err, "walking the path")
+		}
+		if info.IsDir() || !rm(strings.TrimPrefix(p, dbpath)) {
+			return nil
+		}
+
+		r.log.Debug("rm %s", p)
+		err = os.Remove(p)
+		if err != nil {
+			r.log.Error("datadir cleanup: remove %s: %v", p, err)
+		}
+		return nil
+	})
+}
+
+func (r *PhysRestore) writeStat(stat any) error {
+	b, err := json.Marshal(stat)
 	if err != nil {
 		return errors.Wrap(err, "marshal")
 	}
@@ -830,11 +1021,13 @@ func (r *PhysRestore) copyFiles() (stat *s3.DownloadStat, err error) {
 			r.log.Debug("download stat: %s", s)
 		}()
 	}
+
+	setName := pbm.MakeReverseRSMapFunc(r.rsMap)(r.nodeInfo.SetName)
 	cpbuf := make([]byte, 32*1024)
 	for i := len(r.files) - 1; i >= 0; i-- {
 		set := r.files[i]
 		for _, f := range set.Data {
-			src := filepath.Join(set.BcpName, r.nodeInfo.SetName, f.Name+set.Cmpr.Suffix())
+			src := filepath.Join(set.BcpName, setName, f.Name+set.Cmpr.Suffix())
 			if f.Len != 0 {
 				src += fmt.Sprintf(".%d-%d", f.Off, f.Len)
 			}
@@ -894,6 +1087,47 @@ func (r *PhysRestore) copyFiles() (stat *s3.DownloadStat, err error) {
 	return stat, nil
 }
 
+func (r *PhysRestore) getLasOpTime() (primitive.Timestamp, error) {
+	err := r.startMongo("--dbpath", r.dbpath,
+		"--setParameter", "disableLogicalSessionCacheRefresh=true")
+	if err != nil {
+		return primitive.Timestamp{}, errors.Wrap(err, "start mongo")
+	}
+
+	c, err := tryConn(5, time.Minute*5, r.tmpPort, path.Join(r.dbpath, internalMongodLog))
+	if err != nil {
+		return primitive.Timestamp{}, errors.Wrap(err, "connect to mongo")
+	}
+
+	ctx := context.Background()
+
+	res := c.Database("local").Collection("oplog.rs").FindOne(
+		ctx,
+		bson.M{},
+		options.FindOne().SetSort(bson.D{{"ts", -1}}),
+	)
+	if res.Err() != nil {
+		return primitive.Timestamp{}, errors.Wrap(res.Err(), "get oplog entry")
+	}
+	rb, err := res.DecodeBytes()
+	if err != nil {
+		return primitive.Timestamp{}, errors.Wrap(err, "decode oplog entry")
+	}
+	ts := primitive.Timestamp{}
+	var ok bool
+	ts.T, ts.I, ok = rb.Lookup("ts").TimestampOK()
+	if !ok {
+		return ts, errors.Errorf("get the timestamp of record %v", rb)
+	}
+
+	err = shutdown(c, r.dbpath)
+	if err != nil {
+		return ts, errors.Wrap(err, "shutdown mongo")
+	}
+
+	return ts, nil
+}
+
 func (r *PhysRestore) prepareData() error {
 	err := r.startMongo("--dbpath", r.dbpath,
 		"--setParameter", "disableLogicalSessionCacheRefresh=true")
@@ -932,9 +1166,9 @@ func (r *PhysRestore) prepareData() error {
 		return errors.Wrap(err, "insert to replset.minvalid")
 	}
 
-	r.log.Debug("oplogTruncateAfterPoint: %v", r.bcp.LastWriteTS)
+	r.log.Debug("oplogTruncateAfterPoint: %v", r.restoreTS)
 	_, err = c.Database("local").Collection("replset.oplogTruncateAfterPoint").InsertOne(ctx,
-		bson.M{"_id": "oplogTruncateAfterPoint", "oplogTruncateAfterPoint": r.bcp.LastWriteTS},
+		bson.M{"_id": "oplogTruncateAfterPoint", "oplogTruncateAfterPoint": r.restoreTS},
 	)
 	if err != nil {
 		return errors.Wrap(err, "set oplogTruncateAfterPoint")
@@ -983,6 +1217,100 @@ func (r *PhysRestore) recoverStandalone() error {
 	return nil
 }
 
+func (r *PhysRestore) replayOplog(from, to primitive.Timestamp, opChunks []pbm.OplogChunk, stat *pbm.RestoreShardStat) error {
+	err := r.startMongo("--dbpath", r.dbpath,
+		"--setParameter", "disableLogicalSessionCacheRefresh=true")
+	if err != nil {
+		return errors.Wrap(err, "start mongo")
+	}
+
+	c, err := tryConn(5, time.Minute*5, r.tmpPort, path.Join(r.dbpath, internalMongodLog))
+	if err != nil {
+		return errors.Wrap(err, "connect to mongo")
+	}
+
+	ctx := context.Background()
+	_, err = c.Database("local").Collection("system.replset").InsertOne(ctx,
+		pbm.RSConfig{
+			ID:      r.rsConf.ID,
+			CSRS:    r.nodeInfo.IsConfigSrv(),
+			Version: 1,
+			Members: []pbm.RSMember{{
+				ID:           0,
+				Host:         "localhost:" + strconv.Itoa(r.tmpPort),
+				Votes:        1,
+				Priority:     1,
+				BuildIndexes: true,
+			}},
+		},
+	)
+	if err != nil {
+		return errors.Wrapf(err, "upate rs.member host to %s", r.nodeInfo.Me)
+	}
+
+	err = shutdown(c, r.dbpath)
+	if err != nil {
+		return errors.Wrap(err, "shutdown mongo")
+	}
+
+	flags := []string{"--dbpath", r.dbpath,
+		"--setParameter", "disableLogicalSessionCacheRefresh=true",
+		"--setParameter", "takeUnstableCheckpointOnShutdown=true",
+		"--replSet", r.rsConf.ID}
+	if r.nodeInfo.IsConfigSrv() {
+		flags = append(flags, "--configsvr")
+	}
+	err = r.startMongo(flags...)
+	if err != nil {
+		return errors.Wrap(err, "start mongo as rs")
+	}
+
+	c, err = tryConn(5, time.Minute*5, r.tmpPort, path.Join(r.dbpath, internalMongodLog))
+	if err != nil {
+		return errors.Wrap(err, "connect to mongo rs")
+	}
+
+	mgoV, err := pbm.GetMongoVersion(ctx, c)
+	if err != nil || len(mgoV.Version) < 1 {
+		return errors.Wrap(err, "define mongo version")
+	}
+
+	oplogOption := applyOplogOption{
+		start:  &from,
+		end:    &to,
+		unsafe: true,
+	}
+	partial, err := applyOplog(c, opChunks, &oplogOption, r.nodeInfo.IsSharded(),
+		nil, r.setcommittedTxn, r.getcommittedTxn, &stat.Txn,
+		&mgoV, r.stg, r.log)
+	if err != nil {
+		return errors.Wrap(err, "reply oplog")
+	}
+	if len(partial) > 0 {
+		tops := []db.Oplog{}
+		for _, t := range partial {
+			tops = append(tops, t.Oplog...)
+		}
+
+		var b bytes.Buffer
+		err := json.NewEncoder(&b).Encode(tops)
+		if err != nil {
+			return errors.Wrap(err, "encode")
+		}
+		err = r.stg.Save(r.syncPathRS+".partTxn", &b, int64(b.Len()))
+		if err != nil {
+			return errors.Wrap(err, "write partial transactions")
+		}
+	}
+
+	err = shutdown(c, r.dbpath)
+	if err != nil {
+		return errors.Wrap(err, "shutdown mongo")
+	}
+
+	return nil
+}
+
 func (r *PhysRestore) resetRS() error {
 	err := r.startMongo("--dbpath", r.dbpath,
 		"--setParameter", "disableLogicalSessionCacheRefresh=true",
@@ -1007,45 +1335,72 @@ func (r *PhysRestore) resetRS() error {
 		if err != nil {
 			return errors.Wrap(err, "drop config.lockpings")
 		}
-		for id, host := range r.shards {
-			_, err = c.Database("config").Collection("shards").UpdateOne(
-				ctx,
-				bson.D{{"_id", id}},
-				bson.D{
-					{"$set", bson.M{"host": host}},
-				},
-			)
 
-			if err != nil {
-				return errors.Wrapf(err, "update config.shards for %s %s", id, host)
+		cur, err := c.Database("config").Collection("shards").Find(ctx, bson.D{})
+		if err != nil {
+			return errors.WithMessage(err, "find: config.shards")
+		}
+
+		var docs []struct {
+			I string         `bson:"_id"`
+			H string         `bson:"host"`
+			R map[string]any `bson:",inline"`
+		}
+		if err := cur.All(ctx, &docs); err != nil {
+			return errors.WithMessage(err, "decode: config.shards")
+		}
+
+		sMap := r.getShardMapping(r.bcp)
+		mapS := pbm.MakeRSMapFunc(sMap)
+		ms := []mongo.WriteModel{&mongo.DeleteManyModel{Filter: bson.D{}}}
+		for _, doc := range docs {
+			doc.I = mapS(doc.I)
+			doc.H = r.shards[doc.I]
+			ms = append(ms, &mongo.InsertOneModel{Document: doc})
+		}
+
+		_, err = c.Database("config").Collection("shards").BulkWrite(ctx, ms)
+		if err != nil {
+			return errors.Wrap(err, "update config.shards")
+		}
+
+		if len(sMap) != 0 {
+			r.log.Debug("updating router config")
+			if err := updateRouterTables(ctx, c, sMap); err != nil {
+				return errors.WithMessage(err, "update router tables")
 			}
 		}
 	} else {
+		var currS string
+		for s, uri := range r.shards {
+			rs, _, _ := strings.Cut(uri, "/")
+			if rs == r.nodeInfo.SetName {
+				currS = s
+				break
+			}
+		}
+
 		_, err = c.Database("admin").Collection("system.version").UpdateOne(
 			ctx,
 			bson.D{{"_id", "shardIdentity"}},
-			bson.D{
-				{"$set", bson.M{"configsvrConnectionString": r.cfgConn}},
-			},
+			bson.D{{"$set", bson.M{
+				"shardName":                 currS,
+				"configsvrConnectionString": r.cfgConn,
+			}}},
 		)
 		if err != nil {
 			return errors.Wrap(err, "update shardIdentity in admin.system.version")
 		}
 	}
 
-	err = c.Database("config").Collection("cache.collections").Drop(ctx)
+	colls, err := c.Database("config").ListCollectionNames(ctx, bson.D{{"name", bson.M{"$regex": `^cache\.`}}})
 	if err != nil {
-		return errors.Wrap(err, "drop config.cache.collections")
+		return errors.WithMessage(err, "list cache collections")
 	}
-
-	err = c.Database("config").Collection("cache.databases").Drop(ctx)
-	if err != nil {
-		return errors.Wrap(err, "drop config.cache.databases")
-	}
-
-	err = c.Database("config").Collection("cache.chunks.config.system.sessions").Drop(ctx)
-	if err != nil {
-		return errors.Wrap(err, "drop config.cache.chunks.config.system.sessions")
+	for _, coll := range colls {
+		if err := c.Database("config").Collection(coll).Drop(ctx); err != nil {
+			return errors.Wrapf(err, "drop %q", coll)
+		}
 	}
 
 	const retry = 5
@@ -1076,7 +1431,7 @@ func (r *PhysRestore) resetRS() error {
 		},
 	)
 	if err != nil {
-		return errors.Wrapf(err, "upate rs.member host to %s", r.nodeInfo.Me)
+		return errors.Wrapf(err, "update rs.member host to %s", r.nodeInfo.Me)
 	}
 
 	// PITR should be turned off after the physical restore. Otherwise, slicing resumes
@@ -1101,6 +1456,147 @@ func (r *PhysRestore) resetRS() error {
 	return nil
 }
 
+func (r *PhysRestore) getShardMapping(bcp *pbm.BackupMeta) map[string]string {
+	source := make(map[string]string)
+	if bcp != nil && bcp.ShardRemap != nil {
+		for i := range bcp.Replsets {
+			rs := bcp.Replsets[i].Name
+			if s, ok := bcp.ShardRemap[rs]; ok {
+				source[rs] = s
+			}
+		}
+	}
+
+	mapRevRS := pbm.MakeReverseRSMapFunc(r.rsMap)
+	rv := make(map[string]string)
+	for targetS, uri := range r.shards {
+		targetRS, _, _ := strings.Cut(uri, "/")
+		sourceRS := mapRevRS(targetRS)
+		sourceS, ok := source[sourceRS]
+		if !ok {
+			sourceS = sourceRS
+		}
+		if sourceS != targetS {
+			rv[sourceS] = targetS
+		}
+	}
+
+	return rv
+}
+
+// in case restore-to-time isn't specified (external restore w/o backup
+// and --ts provided) the cluster will agree on the latest common ts. Each
+// node will pick the ts of the last event in the oplog. Each replset will
+// pick the oldest ts and put it as the reples ts. And the cluster will
+// pick the oldest ts proposed by replsets. All comms done via storage. Similar
+// to the restore states with proposed ts in *.lastTS files.
+func (r *PhysRestore) agreeCommonRestoreTS() (ts primitive.Timestamp, err error) {
+	cts, err := r.getLasOpTime()
+	if err != nil {
+		return ts, errors.Wrap(err, "define last op time")
+	}
+
+	bts := bytes.NewReader([]byte(
+		fmt.Sprintf("%d:%d,%d", time.Now().Unix(), cts.T, cts.I),
+	))
+	// saving straight for RS as backup for nodes in the RS the same,
+	// hence TS would be the same as well
+	err = r.stg.Save(r.syncPathRS+"."+string(pbm.StatusExtTS), bts, -1)
+	if err != nil {
+		return ts, errors.Wrap(err, "write RS timestamp")
+	}
+
+	if r.nodeInfo.IsClusterLeader() {
+		_, err := r.waitFiles(pbm.StatusExtTS, copyMap(r.syncPathShards), true)
+		if err != nil {
+			return ts, errors.Wrap(err, "wait for shards timestamp")
+		}
+		var mints primitive.Timestamp
+		for sh := range r.syncPathShards {
+			ts, err := r.getTSFromSyncFile(sh)
+			if err != nil {
+				return ts, errors.Wrapf(err, "get timestamp for RS %s", sh)
+			}
+
+			if mints.IsZero() || primitive.CompareTimestamp(ts, mints) == -1 {
+				mints = ts
+			}
+		}
+		bts := bytes.NewReader([]byte(
+			fmt.Sprintf("%d:%d,%d", time.Now().Unix(), mints.T, mints.I),
+		))
+		err = r.stg.Save(r.syncPathCluster+"."+string(pbm.StatusExtTS), bts, -1)
+		if err != nil {
+			return ts, errors.Wrap(err, "write")
+		}
+	}
+
+	_, err = r.waitFiles(pbm.StatusExtTS, map[string]struct{}{r.syncPathCluster: {}}, false)
+	if err != nil {
+		return ts, errors.Wrap(err, "wait for cluster timestamp")
+	}
+
+	ts, err = r.getTSFromSyncFile(r.syncPathCluster)
+	if err != nil {
+		return ts, errors.Wrapf(err, "get cluster timestamp")
+	}
+
+	return ts, nil
+}
+
+func (r *PhysRestore) setcommittedTxn(txn []pbm.RestoreTxn) error {
+	if txn == nil {
+		txn = []pbm.RestoreTxn{}
+	}
+	var b bytes.Buffer
+	err := json.NewEncoder(&b).Encode(txn)
+	if err != nil {
+		return errors.Wrap(err, "encode")
+	}
+	return r.stg.Save(r.syncPathRS+".txn",
+		&b, int64(b.Len()),
+	)
+}
+
+func (r *PhysRestore) getcommittedTxn() (map[string]primitive.Timestamp, error) {
+	shards := copyMap(r.syncPathShards)
+	txn := make(map[string]primitive.Timestamp)
+	for len(shards) > 0 {
+		for f := range shards {
+			dr, err := r.stg.FileStat(f + "." + string(pbm.StatusDone))
+			if err != nil && !errors.Is(err, storage.ErrNotExist) {
+				return nil, errors.Wrapf(err, "check done for <%s>", f)
+			}
+			if err == nil && dr.Size != 0 {
+				delete(shards, f)
+				continue
+			}
+
+			txnr, err := r.stg.SourceReader(f + ".txn")
+			if err != nil && errors.Is(err, storage.ErrNotExist) {
+				continue
+			}
+			if err != nil {
+				return nil, errors.Wrapf(err, "get txns <%s>", f)
+			}
+			txns := []pbm.RestoreTxn{}
+			err = json.NewDecoder(txnr).Decode(&txns)
+			if err != nil {
+				return nil, errors.Wrapf(err, "deconde txns <%s>", f)
+			}
+			for _, t := range txns {
+				if t.State == pbm.TxnCommit {
+					txn[t.ID] = t.Ctime
+				}
+			}
+			delete(shards, f)
+		}
+		time.Sleep(time.Second * 5)
+	}
+
+	return txn, nil
+}
+
 // Tries to connect to mongo n times, timeout is applied for each try.
 // If a try is unsuccessful, it will check the mongo logs and retry if
 // there are no errors or fatals.
@@ -1192,6 +1688,18 @@ func (r *PhysRestore) startMongo(opts ...string) error {
 		err := cmd.Wait()
 		if err != nil {
 			slog.Printf("mongod process: %v, %s", err, errBuf)
+			mlog := path.Join(r.dbpath, internalMongodLog)
+			f, err := os.Open(mlog)
+			if err != nil {
+				slog.Printf("open mongod log %s: %v", mlog, err)
+				return
+			}
+			buf, err := io.ReadAll(f)
+			if err != nil {
+				slog.Printf("read mongod log %s: %v", mlog, err)
+				return
+			}
+			slog.Printf("mongod log:\n%s", buf)
 		}
 	}()
 	return nil
@@ -1238,15 +1746,23 @@ func (r *PhysRestore) init(name string, opid pbm.OPID, l *log.Event) (err error)
 			r.syncPathPeers[fmt.Sprintf("%s/%s/rs.%s/node.%s", pbm.PhysRestoresDir, r.name, r.rsConf.ID, m.Host)] = struct{}{}
 		}
 	}
-	if r.nodeInfo.IsConfigSrv() {
-		sh, err := r.cn.GetShards()
-		if err != nil {
-			return errors.Wrap(err, "get data shards")
-		}
-		r.syncPathDataShards = make(map[string]struct{})
-		for _, s := range sh {
-			r.syncPathDataShards[fmt.Sprintf("%s/%s/rs.%s/rs", pbm.PhysRestoresDir, r.name, s.RS)] = struct{}{}
-		}
+
+	dsh, err := r.cn.ClusterMembers()
+	if err != nil {
+		return errors.Wrap(err, "get  shards")
+	}
+	r.syncPathShards = make(map[string]struct{})
+	for _, s := range dsh {
+		r.syncPathShards[fmt.Sprintf("%s/%s/rs.%s/rs", pbm.PhysRestoresDir, r.name, s.RS)] = struct{}{}
+	}
+
+	sh, err := r.cn.GetShards()
+	if err != nil {
+		return errors.Wrap(err, "get data shards")
+	}
+	r.syncPathDataShards = make(map[string]struct{})
+	for _, s := range sh {
+		r.syncPathDataShards[fmt.Sprintf("%s/%s/rs.%s/rs", pbm.PhysRestoresDir, r.name, s.RS)] = struct{}{}
 	}
 
 	err = r.hb()
@@ -1341,16 +1857,20 @@ func (r *PhysRestore) checkHB(file string) error {
 	return nil
 }
 
-func (r *PhysRestore) setTmpConf() (err error) {
+func (r *PhysRestore) setTmpConf(xopts *pbm.MongodOpts) (err error) {
 	opts := new(pbm.MongodOpts)
-	for _, v := range r.bcp.Replsets {
-		if v.Name == r.nodeInfo.SetName {
-			if v.MongodOpts == nil {
-				return nil
+	opts.Storage = *pbm.NewMongodOptsStorage()
+	if xopts != nil {
+		opts.Storage = xopts.Storage
+	} else if r.bcp != nil {
+		setName := pbm.MakeReverseRSMapFunc(r.rsMap)(r.nodeInfo.SetName)
+		for _, v := range r.bcp.Replsets {
+			if v.Name == setName {
+				if v.MongodOpts != nil {
+					opts.Storage = v.MongodOpts.Storage
+				}
+				break
 			}
-
-			opts.Storage = v.MongodOpts.Storage
-			break
 		}
 	}
 
@@ -1398,9 +1918,10 @@ const bcpDir = "__dir__"
 func (r *PhysRestore) setBcpFiles() (err error) {
 	bcp := r.bcp
 
-	rs := getRS(bcp, r.nodeInfo.SetName)
+	setName := pbm.MakeReverseRSMapFunc(r.rsMap)(r.nodeInfo.SetName)
+	rs := getRS(bcp, setName)
 	if rs == nil {
-		return errors.Errorf("no data in the backup for the replica set %s", r.nodeInfo.SetName)
+		return errors.Errorf("no data in the backup for the replica set %s", setName)
 	}
 
 	targetFiles := make(map[string]bool)
@@ -1441,7 +1962,7 @@ func (r *PhysRestore) setBcpFiles() (err error) {
 		if err != nil {
 			return errors.Wrapf(err, "get source backup")
 		}
-		rs = getRS(bcp, r.nodeInfo.SetName)
+		rs = getRS(bcp, setName)
 	}
 
 	// Directories only. Incremental $backupCusor returns collections that
@@ -1562,11 +2083,10 @@ func (r *PhysRestore) prepareBackup(backupName string) (err error) {
 		return errors.Wrap(err, "get cluster members")
 	}
 
+	mapRevRS := pbm.MakeReverseRSMapFunc(r.rsMap)
 	fl := make(map[string]pbm.Shard, len(s))
-	r.syncPathShards = make(map[string]struct{})
 	for _, rs := range s {
-		fl[rs.RS] = rs
-		r.syncPathShards[fmt.Sprintf("%s/%s/rs.%s/rs", pbm.PhysRestoresDir, r.name, rs.RS)] = struct{}{}
+		fl[mapRevRS(rs.RS)] = rs
 	}
 
 	var nors []string
@@ -1580,9 +2100,10 @@ func (r *PhysRestore) prepareBackup(backupName string) (err error) {
 		return errors.Errorf("extra/unknown replica set found in the backup: %s", strings.Join(nors, ","))
 	}
 
+	setName := mapRevRS(r.nodeInfo.SetName)
 	var ok bool
 	for _, v := range r.bcp.Replsets {
-		if v.Name == r.nodeInfo.SetName {
+		if v.Name == setName {
 			ok = true
 			break
 		}
